@@ -7,7 +7,6 @@ from abc import ABCMeta, abstractmethod
 from typing import Any, Generic, List, Optional, Sequence, TypeVar, Union
 
 import pandas as pd
-from sklearn.base import BaseEstimator
 
 from pytools.api import AllTracker, inheritdoc
 
@@ -37,9 +36,7 @@ __tracker = AllTracker(globals())
 
 
 @inheritdoc(match="[see superclass]")
-class _EstimatorPipelineDF(
-    EstimatorDF, BaseEstimator, Generic[T_FinalEstimatorDF], metaclass=ABCMeta
-):
+class _EstimatorPipelineDF(EstimatorDF, Generic[T_FinalEstimatorDF], metaclass=ABCMeta):
     """
     A data frame enabled pipeline with an optional preprocessing step and a
     mandatory estimator step.
@@ -124,7 +121,6 @@ class _EstimatorPipelineDF(
         X: pd.DataFrame,
         y: Optional[Union[pd.Series, pd.DataFrame]] = None,
         *,
-        feature_sequence: Optional[pd.Index] = None,
         sample_weight: Optional[pd.Series] = None,
         **fit_params,
     ) -> T_Self:
@@ -133,8 +129,6 @@ class _EstimatorPipelineDF(
 
         :param X: input data frame with observations as rows and features as columns
         :param y: an optional series or data frame with one or more outputs
-        :param feature_sequence: the order in which features should be passed to the
-            final estimator (optional)
         :param sample_weight: sample weights for observations, to be passed to the
             final estimator (optional)
         :param fit_params: additional keyword parameters as required by specific
@@ -145,22 +139,6 @@ class _EstimatorPipelineDF(
 
         X_preprocessed: pd.DataFrame = self._pre_fit_transform(X, y, **fit_params)
 
-        if feature_sequence is not None:
-            if not feature_sequence.is_unique:
-                raise ValueError("arg feature_sequence contains duplicate values")
-            features = X_preprocessed.columns
-            if not features.is_unique:
-                raise ValueError(
-                    "arg X has columns with duplicate names after preprocessing"
-                )
-            features_reordered = feature_sequence.intersection(features, sort=False)
-            if len(features_reordered) < len(features):
-                raise ValueError(
-                    "arg feature_sequence misses features: "
-                    f"{', '.join(features.difference(feature_sequence))}"
-                )
-            X_preprocessed = X_preprocessed.reindex(columns=features_reordered)
-
         if sample_weight is None:
             self.final_estimator.fit(X_preprocessed, y, **fit_params)
         else:
@@ -220,14 +198,14 @@ class LearnerPipelineDF(
 
     # noinspection PyPep8Naming
     def predict(
-        self, X: pd.DataFrame, **predict_params
+        self, X: pd.DataFrame, **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
         return self.final_estimator.predict(self._pre_transform(X), **predict_params)
 
     # noinspection PyPep8Naming
     def fit_predict(
-        self, X: pd.DataFrame, y: pd.Series, **fit_params
+        self, X: pd.DataFrame, y: pd.Series, **fit_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
         return self.final_estimator.fit_predict(
@@ -337,14 +315,14 @@ class ClassifierPipelineDF(
 
     # noinspection PyPep8Naming
     def predict_proba(
-        self, X: pd.DataFrame, **predict_params
+        self, X: pd.DataFrame, **predict_params: Any
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """[see superclass]"""
         return self.classifier.predict_proba(self._pre_transform(X), **predict_params)
 
     # noinspection PyPep8Naming
     def predict_log_proba(
-        self, X: pd.DataFrame, **predict_params
+        self, X: pd.DataFrame, **predict_params: Any
     ) -> Union[pd.DataFrame, List[pd.DataFrame]]:
         """[see superclass]"""
         return self.classifier.predict_log_proba(
@@ -353,7 +331,7 @@ class ClassifierPipelineDF(
 
     # noinspection PyPep8Naming
     def decision_function(
-        self, X: pd.DataFrame, **predict_params
+        self, X: pd.DataFrame, **predict_params: Any
     ) -> Union[pd.Series, pd.DataFrame]:
         """[see superclass]"""
         return self.classifier.decision_function(
