@@ -3,7 +3,7 @@ Extended versions of all Scikit-Learn transformers with enhanced E2E support for
 frames.
 """
 
-from .. import __sklearn_0_22__, __sklearn_0_23__, __sklearn_version__
+from .. import __sklearn_0_22__, __sklearn_0_23__, __sklearn_0_24__, __sklearn_version__
 from ._transformation import *
 
 if __sklearn_version__ >= __sklearn_0_22__:
@@ -11,3 +11,6 @@ if __sklearn_version__ >= __sklearn_0_22__:
 
 if __sklearn_version__ >= __sklearn_0_23__:
     from ._transformation_v0_23 import *
+
+if __sklearn_version__ >= __sklearn_0_24__:
+    from ._transformation_v0_24 import *
