@@ -13,7 +13,7 @@ The wrappers also implement the additional column attributes introduced by `skle
 
 import inspect
 import logging
-from abc import ABCMeta, abstractmethod
+from abc import ABCMeta
 from functools import update_wrapper
 from typing import (
     Any,
@@ -36,7 +36,6 @@ from weakref import WeakValueDictionary
 
 import numpy as np
 import pandas as pd
-import sklearn.utils.metaestimators as sklearn_meta
 from sklearn.base import (
     BaseEstimator,
     ClassifierMixin,
@@ -48,7 +47,6 @@ from sklearn.base import (
 from pytools.api import AllTracker, inheritdoc, public_module_prefix
 from pytools.meta import compose_meta
 
-from ._adapter import LearnerNPDF
 from sklearndf import ClassifierDF, EstimatorDF, LearnerDF, RegressorDF, TransformerDF
 
 log = logging.getLogger(__name__)
@@ -81,12 +79,12 @@ T_NativeLearner = TypeVar("T_NativeLearner", RegressorMixin, ClassifierMixin)
 T_NativeRegressor = TypeVar("T_NativeRegressor", bound=RegressorMixin)
 T_NativeClassifier = TypeVar("T_NativeClassifier", bound=ClassifierMixin)
 
+# noinspection PyTypeChecker
 T_EstimatorWrapperDF = TypeVar("T_EstimatorWrapperDF", bound="EstimatorWrapperDF")
 T_TransformerWrapperDF = TypeVar("T_TransformerWrapperDF", bound="TransformerWrapperDF")
 T_RegressorWrapperDF = TypeVar("T_RegressorWrapperDF", bound="RegressorWrapperDF")
 T_ClassifierWrapperDF = TypeVar("T_ClassifierWrapperDF", bound="ClassifierWrapperDF")
 
-T_LearnerDF = TypeVar("T_LearnerDF", bound="LearnerDF")
 
 #
 # Ensure all symbols introduced below are included in __all__
@@ -149,17 +147,15 @@ class EstimatorWrapperDF(
         if fitted_delegate_context is None:
             # create a new delegate estimator with the given parameters
             # noinspection PyProtectedMember
-            _native_estimator = type(self).__wrapped__(*args, **kwargs)
+            self._native_estimator = type(self).__wrapped__(*args, **kwargs)
             self._reset_fit()
         else:
             (
-                _native_estimator,
+                self._native_estimator,
                 self._features_in,
                 self._n_outputs,
             ) = fitted_delegate_context
 
-        self._native_estimator = _native_estimator
-
         self._validate_delegate_estimator()
 
     def __new__(cls: Type[T], *args, **kwargs: Any) -> T:
@@ -180,14 +176,6 @@ class EstimatorWrapperDF(
         """
         return self._native_estimator
 
-    @property
-    def _estimator_type(self) -> Optional[str]:
-        try:
-            # noinspection PyProtectedMember
-            return self.native_estimator._estimator_type
-        except AttributeError:
-            return None
-
     @classmethod
     def from_fitted(
         cls: Type[T_EstimatorWrapperDF],
@@ -269,8 +257,8 @@ class EstimatorWrapperDF(
     ) -> T_NativeEstimator:
         # noinspection PyUnresolvedReferences
         return self._native_estimator.fit(
-            self._prepare_X_for_delegate(X),
-            self._prepare_y_for_delegate(y),
+            self._convert_X_for_delegate(X),
+            self._convert_y_for_delegate(y),
             **fit_params,
         )
 
@@ -288,28 +276,16 @@ class EstimatorWrapperDF(
 
     # noinspection PyPep8Naming
     def _check_parameter_types(
-        self,
-        X: pd.DataFrame,
-        y: Optional[Union[pd.Series, pd.DataFrame]],
-        *,
-        expected_columns: pd.Index = None,
+        self, X: pd.DataFrame, y: Optional[Union[pd.Series, pd.DataFrame]]
     ) -> None:
         if not isinstance(X, pd.DataFrame):
             raise TypeError("arg X must be a DataFrame")
-
         if self.is_fitted:
             EstimatorWrapperDF._verify_df(
-                df_name="arg X",
-                df=X,
-                expected_columns=(
-                    self.feature_names_in_
-                    if expected_columns is None
-                    else expected_columns
-                ),
+                df_name="X argument", df=X, expected_columns=self.feature_names_in_
             )
-
         if y is not None and not isinstance(y, (pd.Series, pd.DataFrame)):
-            raise TypeError("arg y must be None, or a pandas series or data frame")
+            raise TypeError("arg y must be None, or a pandas Series or DataFrame")
 
     @staticmethod
     def _verify_df(
@@ -351,21 +327,8 @@ class EstimatorWrapperDF(
             )
 
     # noinspection PyPep8Naming
-    def _prepare_X_for_delegate(self, X: pd.DataFrame) -> Any:
-        # convert X before passing it to the delegate estimator
-        return self._adjust_X_type_for_delegate(self._adjust_X_columns_for_delegate(X))
-
-    def _prepare_y_for_delegate(
-        self, y: Optional[Union[pd.Series, pd.DataFrame]]
-    ) -> Any:
-        return self._adjust_y_type_for_delegate(y)
-
-    # noinspection PyPep8Naming
-    def _adjust_X_columns_for_delegate(self, X: pd.DataFrame) -> pd.DataFrame:
-        # make sure columns of X are aligned with frame used to fit this estimator
-
+    def _convert_X_for_delegate(self, X: pd.DataFrame) -> Any:
         if not self.is_fitted:
-            # return X unchanged if estimator is not fitted yet
             return X
 
         features_in = self._get_features_in()
@@ -374,21 +337,10 @@ class EstimatorWrapperDF(
         else:
             return X.reindex(columns=features_in, copy=False)
 
-    # noinspection PyPep8Naming
-    def _adjust_X_type_for_delegate(
-        self, X: pd.DataFrame, *, to_numpy: Optional[bool] = None
+    def _convert_y_for_delegate(
+        self, y: Optional[Union[pd.Series, pd.DataFrame]]
     ) -> Any:
-        # convert X before passing it to the delegate estimator
-        return X.values if to_numpy else X
-
-    def _adjust_y_type_for_delegate(
-        self,
-        y: Optional[Union[pd.Series, pd.DataFrame]],
-        *,
-        to_numpy: Optional[bool] = None,
-    ) -> Any:
-        # convert y before passing it to the delegate estimator
-        return y if y is None or not to_numpy else y.values
+        return y
 
     def _make_verbose_exception(self, method: str, cause: Exception) -> Exception:
         verbose_message = f"{type(self).__name__}.{method}: {cause}"
@@ -480,7 +432,9 @@ class TransformerWrapperDF(
     # noinspection PyPep8Naming
     def inverse_transform(self, X: pd.DataFrame) -> pd.DataFrame:
         """[see superclass]"""
-        self._check_parameter_types(X, None, expected_columns=self.feature_names_out_)
+        self._reset_fit()
+
+        self._check_parameter_types(X, None)
 
         transformed = self._inverse_transform(X)
 
@@ -495,29 +449,6 @@ class TransformerWrapperDF(
         finally:
             self._features_original = None
 
-    # noinspection PyPep8Naming
-    def _prepare_X_for_delegate(self, X: pd.DataFrame, *, inverse: bool = False) -> Any:
-        return self._adjust_X_type_for_delegate(
-            self._adjust_X_columns_for_delegate(X, inverse=inverse),
-            to_numpy=inverse or None,
-        )
-
-    # noinspection PyPep8Naming
-    def _adjust_X_columns_for_delegate(
-        self, X: pd.DataFrame, *, inverse: Optional[bool] = None
-    ) -> pd.DataFrame:
-        if inverse:
-            # when converting X for an inverse transform, ensure the data frame is
-            # aligned with the output features, and convert the data frame to a
-            # numpy array
-            features_out = self.feature_names_out_
-            if X.columns.is_(features_out):
-                return X
-            else:
-                return X.reindex(columns=features_out, copy=False)
-        else:
-            return super()._adjust_X_columns_for_delegate(X)
-
     @staticmethod
     def _transformed_to_df(
         transformed: Union[pd.DataFrame, np.ndarray], index: pd.Index, columns: pd.Index
@@ -537,28 +468,22 @@ class TransformerWrapperDF(
     # noinspection PyPep8Naming
     def _transform(self, X: pd.DataFrame) -> np.ndarray:
         # noinspection PyUnresolvedReferences
-        return self.native_estimator.transform(self._prepare_X_for_delegate(X))
+        return self.native_estimator.transform(self._convert_X_for_delegate(X))
 
     # noinspection PyPep8Naming
     def _fit_transform(
         self, X: pd.DataFrame, y: Optional[pd.Series], **fit_params
     ) -> np.ndarray:
         return self.native_estimator.fit_transform(
-            self._prepare_X_for_delegate(X),
-            self._prepare_y_for_delegate(y),
+            self._convert_X_for_delegate(X),
+            self._convert_y_for_delegate(y),
             **fit_params,
         )
 
     # noinspection PyPep8Naming
     def _inverse_transform(self, X: pd.DataFrame) -> np.ndarray:
-        try:
-            inverse_transform_fn = self.native_estimator.inverse_transform
-        except AttributeError:
-            raise NotImplementedError(
-                f"{type(self).__name__} does not implement method inverse_transform()"
-            )
-
-        return inverse_transform_fn(self._prepare_X_for_delegate(X, inverse=True))
+        # noinspection PyUnresolvedReferences
+        return self.native_estimator.inverse_transform(self._convert_X_for_delegate(X))
 
 
 @inheritdoc(match="[see superclass]")
@@ -590,7 +515,7 @@ class LearnerWrapperDF(
         return self._prediction_to_series_or_frame(
             X,
             self.native_estimator.predict(
-                self._prepare_X_for_delegate(X), **predict_params
+                self._convert_X_for_delegate(X), **predict_params
             ),
         )
 
@@ -609,8 +534,8 @@ class LearnerWrapperDF(
             result = self._prediction_to_series_or_frame(
                 X,
                 self.native_estimator.fit_predict(
-                    self._prepare_X_for_delegate(X),
-                    self._prepare_y_for_delegate(y),
+                    self._convert_X_for_delegate(X),
+                    self._convert_y_for_delegate(y),
                     **fit_params,
                 ),
             )
@@ -637,8 +562,8 @@ class LearnerWrapperDF(
             raise TypeError("arg sample_weight must be None or a Series")
 
         return self.native_estimator.score(
-            self._prepare_X_for_delegate(X),
-            self._prepare_y_for_delegate(y),
+            self._convert_X_for_delegate(X),
+            self._convert_y_for_delegate(y),
             sample_weight,
         )
 
@@ -719,7 +644,7 @@ class ClassifierWrapperDF(
         return self._prediction_with_class_labels(
             X,
             self.native_estimator.predict_proba(
-                self._prepare_X_for_delegate(X), **predict_params
+                self._convert_X_for_delegate(X), **predict_params
             ),
         )
 
@@ -737,7 +662,7 @@ class ClassifierWrapperDF(
         return self._prediction_with_class_labels(
             X,
             self.native_estimator.predict_log_proba(
-                self._prepare_X_for_delegate(X), **predict_params
+                self._convert_X_for_delegate(X), **predict_params
             ),
         )
 
@@ -755,7 +680,7 @@ class ClassifierWrapperDF(
         return self._prediction_with_class_labels(
             X,
             self.native_estimator.decision_function(
-                self._prepare_X_for_delegate(X), **predict_params
+                self._convert_X_for_delegate(X), **predict_params
             ),
         )
 
@@ -777,8 +702,9 @@ class ClassifierWrapperDF(
         if classes is None:
             classes = getattr(self.native_estimator, "classes_", None)
 
-        if isinstance(y, pd.DataFrame):
-            return y.set_axis(classes, axis=1, inplace=False)
+        if isinstance(y, pd.DataFrame) or isinstance(y, pd.Series):
+            # if we already have a series or data frame, return it unchanged
+            return y
         elif isinstance(y, np.ndarray):
             if len(y) == len(X):
                 # predictions of probabilities are usually provided as a NumPy array
@@ -826,20 +752,11 @@ class MetaEstimatorWrapperDF(
 
     def _validate_delegate_estimator(self) -> None:
         def _unwrap_estimator(estimator: BaseEstimator) -> BaseEstimator:
-            native_estimator = (
+            return (
                 estimator.native_estimator
-                if isinstance(estimator, EstimatorWrapperDF)
+                if isinstance(estimator, EstimatorDF)
                 else estimator
             )
-            # noinspection PyProtectedMember
-            if isinstance(
-                native_estimator, (EstimatorDF, sklearn_meta._BaseComposition)
-            ) or not isinstance(native_estimator, (RegressorMixin, ClassifierMixin)):
-                raise TypeError(
-                    "sklearndf meta-estimators only accept simple regressors and "
-                    f"classifiers, but got: {type(estimator).__name__}"
-                )
-            return cast(BaseEstimator, native_estimator)
 
         delegate_estimator = self.native_estimator
 
@@ -863,216 +780,51 @@ class MetaEstimatorWrapperDF(
 #
 
 
-# noinspection PyPep8Naming
-@inheritdoc(match="""[see superclass]""")
 class StackingEstimatorWrapperDF(
-    LearnerWrapperDF[T_NativeLearner],
+    EstimatorWrapperDF[T_NativeEstimator],
     # note: MetaEstimatorMixin is the first public class in the mro of _BaseStacking
     # MetaEstimatorMixin <-- _BaseHeterogeneousEnsemble <-- _BaseStacking
     MetaEstimatorMixin,
-    Generic[T_NativeLearner],
+    Generic[T_NativeEstimator],
     metaclass=ABCMeta,
 ):
     """
-    Abstract base class of wrappers for estimators implementing
-    :class:`sklearn.ensemble._stacking._BaseStacking`.
+    Abstract base class wrapping around estimators implementing
+    :class:`sklearn.ensemble._stacking._BaseStacking`. The stacking estimator will call
+    the methods of the embedded estimator using a modified copy of the `X` and `y`
+    parameters, so we need to make sure that these are converted back to data frames.
 
-    The stacking estimator will delegate to embedded estimators; this wrapper ensures
-    the required conversions from and to numpy arrays as the native stacking estimator
-    invokes the embedded estimators.
+    This class covers the following cases used in sklearn:
+    - one (optional) inner estimator in attribute `final_estimator`
+    - multiple stacked estimators in attribute `estimators`, as name estimator pairs
     """
 
-    def fit(
-        self: T_Self,
-        X: pd.DataFrame,
-        y: Optional[Union[pd.Series, pd.DataFrame]] = None,
-        **fit_params: Any,
-    ) -> T_Self:
-        """[see superclass]"""
-
-        self: StackingEstimatorWrapperDF
-
-        class _ColumnNameFn:
-            # noinspection PyMethodParameters
-            def __call__(self_) -> Sequence[str]:
-                return self._get_final_estimator_features_in()
-
-            def __deepcopy__(self, memo=None) -> Any:
-                # prevent a deep copy of this callable, to preserve reference to
-                # stacking estimator being fitted
-                return self
-
-        native: T_NativeEstimator = self.native_estimator
-        estimators: Sequence[Tuple[str, BaseEstimator]] = native.estimators
-        final_estimator: BaseEstimator = native.final_estimator
-
-        try:
-            native.estimators = [
-                (
-                    name,
-                    self._make_stackable_learner_df(estimator)
-                    if isinstance(estimator, LearnerDF)
-                    else estimator,
+    def _validate_delegate_estimator(self) -> None:
+        def _unwrap_estimator(estimator: BaseEstimator) -> Optional[BaseEstimator]:
+            if estimator is None:
+                return None
+            else:
+                return (
+                    estimator.native_estimator
+                    if isinstance(estimator, EstimatorDF)
+                    else estimator
                 )
-                for name, estimator in native.estimators
+
+        delegate_estimator = self.native_estimator
+
+        # note: as final_estimator is optional, _unwrap_estimator will return None
+        #       attribute "named_estimators_" is constructed based off estimators
+
+        if hasattr(delegate_estimator, "final_estimator"):
+            delegate_estimator.final_estimator = _unwrap_estimator(
+                delegate_estimator.final_estimator
+            )
+
+        if hasattr(delegate_estimator, "estimators"):
+            delegate_estimator.estimators = [
+                (name, _unwrap_estimator(estimator))
+                for name, estimator in delegate_estimator.estimators
             ]
-            native.final_estimator = self._make_learner_np_df(
-                delegate=native.final_estimator or self._make_default_final_estimator(),
-                column_names=_ColumnNameFn(),
-            )
-
-            return super().fit(X, y, **fit_params)
-
-        finally:
-            native.estimators = estimators
-            native.final_estimator = final_estimator
-
-    def fit_predict(
-        self, X: pd.DataFrame, y: pd.Series, **fit_params: Any
-    ) -> Union[pd.Series, pd.DataFrame]:
-        """[see superclass]"""
-        self.final_estimator.column_names = self._get_final_estimator_features_in
-
-        return super().fit_predict(X, y, **fit_params)
-
-    @abstractmethod
-    def _make_stackable_learner_df(self, learner: LearnerDF) -> "_StackableLearnerDF":
-        pass
-
-    @abstractmethod
-    def _make_learner_np_df(
-        self, delegate: LearnerDF, column_names: Callable[[], Sequence[str]]
-    ) -> LearnerNPDF:
-        pass
-
-    def _get_estimators_features_out(self) -> List[str]:
-        return [name for name, estimator in self.estimators if estimator != "drop"]
-
-    def _get_final_estimator_features_in(self) -> List[str]:
-        names = self._get_estimators_features_out()
-        if self.passthrough:
-            return [*names, *self.estimators_[0].feature_names_in_]
-        else:
-            return names
-
-
-# noinspection PyPep8Naming
-@inheritdoc(match="""[see superclass]""")
-class _StackableLearnerDF(LearnerDF, Generic[T_LearnerDF]):
-    """
-    Returns numpy arrays from all prediction functions, instead of pandas series or
-    data frames.
-
-    For use in stacking estimators that forward the predictions of multiple learners to
-    one final learner.
-    """
-
-    def __init__(self, delegate: T_LearnerDF) -> None:
-        self.delegate = delegate
-
-    @property
-    def is_fitted(self) -> bool:
-        """[see superclass]"""
-        return self.delegate.is_fitted
-
-    def fit(
-        self: T_Self, X: pd.DataFrame, y: np.ndarray = None, **fit_params: Any
-    ) -> T_Self:
-        """[see superclass]"""
-        self: _StackableLearnerDF
-
-        self.delegate.fit(X, self._convert_y_to_series(X, y), **fit_params)
-        return self
-
-    def predict(self, X: pd.DataFrame, **predict_params: Any) -> np.ndarray:
-        """[see superclass]"""
-        return self.delegate.predict(X, **predict_params).values
-
-    def fit_predict(
-        self, X: pd.DataFrame, y: np.ndarray, **fit_params: Any
-    ) -> np.ndarray:
-        """[see superclass]"""
-        return self.delegate.fit_predict(
-            X, self._convert_y_to_series(X, y), **fit_params
-        ).values
-
-    def score(
-        self, X: pd.DataFrame, y: np.ndarray, sample_weight: Optional[pd.Series] = None
-    ) -> float:
-        """[see superclass]"""
-        return self.delegate.score(X, self._convert_y_to_series(X, y), sample_weight)
-
-    def _get_features_in(self) -> pd.Index:
-        return self.delegate._get_features_in()
-
-    def _get_n_outputs(self) -> int:
-        return self.delegate._get_n_outputs()
-
-    @staticmethod
-    def _convert_y_to_series(
-        X: pd.DataFrame, y: Optional[np.ndarray]
-    ) -> Optional[pd.Series]:
-        if y is None:
-            return y
-        if not isinstance(y, np.ndarray):
-            raise TypeError(
-                f"expected numpy array for arg y but got a {type(y).__name__}"
-            )
-        if y.ndim != 1:
-            raise TypeError(
-                f"expected 1-d numpy array for arg y but got a {y.ndim}-d array"
-            )
-        if len(y) != len(X):
-            raise ValueError(
-                "args X and y have different lengths: "
-                f"len(X)={len(X)} and len(y)={len(y)}"
-            )
-        return pd.Series(y, index=X.index)
-
-    @staticmethod
-    def _convert_prediction_to_numpy(
-        prediction: Union[pd.DataFrame, List[pd.DataFrame]]
-    ) -> Union[np.ndarray, List[np.ndarray]]:
-        if isinstance(prediction, list):
-            return [proba.values for proba in prediction]
-        else:
-            return prediction.values
-
-
-# noinspection PyPep8Naming
-@inheritdoc(match="""[see superclass]""")
-class _StackableClassifierDF(_StackableLearnerDF[ClassifierDF], ClassifierDF):
-    """[see superclass]"""
-
-    @property
-    def classes_(self) -> Sequence[Any]:
-        """[see superclass]"""
-        return self.delegate.classes_
-
-    def predict_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[np.ndarray, List[np.ndarray]]:
-        """[see superclass]"""
-        return self._convert_prediction_to_numpy(
-            self.delegate.predict_proba(X, **predict_params)
-        )
-
-    def predict_log_proba(
-        self, X: pd.DataFrame, **predict_params: Any
-    ) -> Union[np.ndarray, List[np.ndarray]]:
-        """[see superclass]"""
-        return self._convert_prediction_to_numpy(
-            self.delegate.predict_log_proba(X, **predict_params)
-        )
-
-    def decision_function(self, X: pd.DataFrame, **predict_params: Any) -> np.ndarray:
-        """[see superclass]"""
-        return self.delegate.decision_function(X, **predict_params).values
-
-
-@inheritdoc(match="""[see superclass]""")
-class _StackableRegressorDF(_StackableLearnerDF[RegressorDF], RegressorDF):
-    """[see superclass]"""
 
 
 #
@@ -1422,7 +1174,6 @@ def _make_alias(
         )
         function.__doc__ = f"See :meth:`{full_name}`"
         return function
-
     elif inspect.isdatadescriptor(delegate):
         # noinspection PyShadowingNames
         return property(
@@ -1431,7 +1182,6 @@ def _make_alias(
             fdel=lambda self: delegate.__delete__(self._native_estimator),
             doc=f"See documentation of :class:`{class_name}`.",
         )
-
     else:
         return None
 
