@@ -4,17 +4,16 @@ from typing import Type
 import numpy as np
 import pandas as pd
 import pytest
-from sklearn.base import is_classifier
 from sklearn.multioutput import ClassifierChain, MultiOutputClassifier
 
 import sklearndf.classification as classification
 from sklearndf import ClassifierDF
-from test.sklearndf import check_expected_not_fitted_error, list_classes
+from test.sklearndf import check_expected_not_fitted_error, iterate_classes
 
-CLASSIFIERS_TO_TEST = list_classes(
+CLASSIFIERS_TO_TEST = iterate_classes(
     from_modules=classification,
     matching=r".*DF",
-    excluding=[ClassifierDF.__name__, r".*WrapperDF"],
+    excluding=[ClassifierDF.__name__, r".*WrapperDF", r"^_"],
 )
 
 
@@ -36,14 +35,23 @@ CLASSIFIER_INIT_PARAMETERS = {
     },
     "StackingClassifierDF": {
         "estimators": (
-            ("Forest", classification.RandomForestClassifierDF(max_depth=5)),
-            ("Logit", classification.LogisticRegressionCVDF()),
+            ("Forest", classification.RandomForestClassifierDF()),
+            ("SVC", classification.SVCDF()),
             ("AdaBoost", classification.AdaBoostClassifierDF()),
         )
     },
 }
 
 
+@pytest.mark.parametrize(argnames="sklearndf_cls", argvalues=CLASSIFIERS_TO_TEST)
+def test_wrapped_constructor(sklearndf_cls: Type[ClassifierDF]) -> None:
+    """ Test standard constructor of wrapped sklearn classifiers """
+    # noinspection PyArgumentList
+    _: ClassifierDF = sklearndf_cls(
+        **CLASSIFIER_INIT_PARAMETERS.get(sklearndf_cls.__name__, {})
+    )
+
+
 @pytest.mark.parametrize(argnames="sklearndf_cls", argvalues=CLASSIFIERS_TO_TEST)
 def test_wrapped_fit_predict(
     sklearndf_cls: Type[ClassifierDF],
@@ -58,8 +66,6 @@ def test_wrapped_fit_predict(
         **CLASSIFIER_INIT_PARAMETERS.get(sklearndf_cls.__name__, {})
     )
 
-    assert is_classifier(classifier)
-
     is_chain = isinstance(classifier.native_estimator, ClassifierChain)
 
     is_multi_output = isinstance(classifier.native_estimator, MultiOutputClassifier)
@@ -113,7 +119,7 @@ def test_wrapped_fit_predict(
                 assert len(prediction) == len(iris_target_sr)
                 assert prediction.shape == (len(iris_target_sr), len(classes))
                 # check correct labels are set as columns
-                assert set(prediction.columns) == classes
+                assert classes == set(prediction.columns)
         else:
             with pytest.raises(NotImplementedError):
                 method(X=iris_features)
